@@ -3,7 +3,6 @@ import logging
 from os import path
 from uuid import uuid4
 
-import h5py
 import satnogsdecoders
 from django.conf import settings
 from django.contrib.auth import get_user_model
@@ -13,18 +12,15 @@ from django.core.validators import MaxLengthValidator, MaxValueValidator, \
     MinLengthValidator, MinValueValidator, URLValidator
 from django.db import models
 from django.db.models import OuterRef, Subquery
-from django.db.models.signals import post_save, pre_save
 from django.utils.timezone import now
 from django_countries.fields import CountryField
 from markdown import markdown
 from shortuuidfield import ShortUUIDField
 
-from db.base.helpers import gridsquare
-
 LOGGER = logging.getLogger('db')
 
 DATA_SOURCES = ['manual', 'network', 'sids']
-SATELLITE_STATUS = ['alive', 'dead', 're-entered']
+SATELLITE_STATUS = ['alive', 'dead', 'future', 're-entered']
 TRANSMITTER_STATUS = ['active', 'inactive', 'invalid']
 TRANSMITTER_TYPE = ['Transmitter', 'Transceiver', 'Transponder']
 SERVICE_TYPE = [
@@ -62,35 +58,6 @@ def _name_payload_frame(instance, filename):  # pylint: disable=W0613
     return path.join(folder, filename)
 
 
-def _gen_observer(sender, instance, created, **kwargs):  # pylint: disable=W0613
-    post_save.disconnect(_gen_observer, sender=DemodData)
-    try:
-        qth = gridsquare(instance.lat, instance.lng)
-    except Exception:  # pylint: disable=W0703
-        instance.observer = 'Unknown'
-    else:
-        instance.observer = '{0}-{1}'.format(instance.station, qth)
-    instance.save()
-    post_save.connect(_gen_observer, sender=DemodData)
-
-
-def _set_is_decoded(sender, instance, **kwargs):  # pylint: disable=W0613
-    """Returns true if payload_decoded has data"""
-    instance.is_decoded = instance.payload_decoded != ''
-
-
-def _extract_network_obs_id(sender, instance, created, **kwargs):  # pylint: disable=W0613
-    post_save.disconnect(_extract_network_obs_id, sender=Artifact)
-    try:
-        with h5py.File(instance.artifact_file, 'r') as h5_file:
-            instance.network_obs_id = h5_file.attrs["observation_id"]
-    except OSError as error:
-        LOGGER.warning(error)
-
-    instance.save()
-    post_save.connect(_extract_network_obs_id, sender=Artifact)
-
-
 class Mode(models.Model):
     """A satellite transmitter RF mode. For example: FM"""
     name = models.CharField(max_length=25, unique=True)
@@ -115,6 +82,7 @@ class Operator(models.Model):
 class Satellite(models.Model):
     """Model for all the satellites."""
     norad_cat_id = models.PositiveIntegerField()
+    norad_follow_id = models.PositiveIntegerField(blank=True, null=True)
     name = models.CharField(max_length=45)
     names = models.TextField(blank=True)
     description = models.TextField(blank=True)
@@ -125,9 +93,6 @@ class Satellite(models.Model):
         validators=[URLValidator(schemes=['http', 'https'], regex=URL_REGEX)]
     )
     image = models.ImageField(upload_to='satellites', blank=True, help_text='Ideally: 250x250')
-    tle1 = models.CharField(max_length=200, blank=True)
-    tle2 = models.CharField(max_length=200, blank=True)
-    tle_source = models.CharField(max_length=300, blank=True)
     status = models.CharField(
         choices=list(zip(SATELLITE_STATUS, SATELLITE_STATUS)), max_length=10, default='alive'
     )
@@ -219,28 +184,6 @@ class Satellite(models.Model):
         approved_count = Transmitter.objects.filter(satellite=self.id).count()
         return approved_count
 
-    @property
-    def tle_redistributable(self):
-        """Returns True if re-distribution of the TLE is allowed, False otherwise
-
-        :returns: True if re-distribution of the TLE is allowed, False otherwise
-        """
-        return self.tle_source in settings.TLE_SOURCES_REDISTRIBUTABLE
-
-    @property
-    def latest_tle(self):
-        """Returns the latest TLE for this Satellite
-
-        :returns: dict with the latest TLE, it's source and whether redistribution is allowed
-        """
-        return {
-            'source': self.tle_source,
-            'norad_cat_id': self.norad_cat_id,
-            'tle1': self.tle1,
-            'tle2': self.tle2,
-            'redistributable': self.tle_redistributable
-        }
-
     @property
     def latest_data(self):
         """Returns the latest DemodData for this Satellite
@@ -560,15 +503,8 @@ class Tle(models.Model):
 
     class Meta:
         ordering = ['-updated']
-        constraints = [
-            models.UniqueConstraint(
-                fields=['tle1', 'tle2', 'tle_source', 'satellite'],
-                name='unique_entry_from_source_for_satellite'
-            ),
-        ]
         indexes = [
             models.Index(fields=['-updated']),
-            models.Index(fields=['tle1', 'tle2', 'tle_source', 'satellite'])
         ]
         permissions = [('access_all_tles', 'Access all TLEs')]
 
@@ -582,41 +518,16 @@ class Tle(models.Model):
         return [str(self.tle0), str(self.tle1), str(self.tle2)]
 
 
-class LatestTleManager(models.Manager):  # pylint: disable=R0903
-    """Django Manager for latest Tle objects"""
-    def get_queryset(self):
-        """Returns query of latest Tle
-
-        :returns: the latest Tle for each Satellite
-        """
-        subquery = Tle.objects.filter(satellite=OuterRef('satellite')).order_by('-updated')
-        return super().get_queryset().filter(pk=Subquery(subquery.values('pk')[:1]))
-
-
-class LatestDistributableTleManager(models.Manager):  # pylint: disable=R0903
-    """Django Manager for latest Tle objects"""
-    def get_queryset(self):
-        """Returns query of latest Tle
-
-        :returns: the latest Tle for each Satellite
-        """
-        subquery = Tle.objects.filter(tle_source__in=settings.TLE_SOURCES_REDISTRIBUTABLE
-                                      ).filter(satellite=OuterRef('satellite')
-                                               ).order_by('-updated')
-        return super().get_queryset().filter(pk=Subquery(subquery.values('pk')[:1]))
-
-
-class LatestTle(Tle):
-    """LatestTle is the latest entry of a Satellite Tle objects
-    """
-    objects = LatestDistributableTleManager()
-    all_latest_tles = LatestTleManager()
-
-    class Meta:
-        proxy = True
-
-    def __str__(self):
-        return '{:d} - {:s}'.format(self.id, self.tle0)
+class LatestTleSet(models.Model):
+    """LatestTleSet holds the latest entry of a Satellite Tle Set"""
+    satellite = models.OneToOneField(
+        Satellite, related_name='latest_tle_set', on_delete=models.CASCADE
+    )
+    latest = models.ForeignKey(Tle, null=True, related_name='latest', on_delete=models.SET_NULL)
+    latest_distributable = models.ForeignKey(
+        Tle, null=True, related_name='latest_distributable', on_delete=models.SET_NULL
+    )
+    last_modified = models.DateTimeField(auto_now=True)
 
 
 class Telemetry(models.Model):
@@ -697,10 +608,6 @@ class DemodData(models.Model):
             return None
 
 
-post_save.connect(_gen_observer, sender=DemodData)
-pre_save.connect(_set_is_decoded, sender=DemodData)
-
-
 class ExportedFrameset(models.Model):
     """Model for exported frames."""
     created = models.DateTimeField(auto_now_add=True)
@@ -720,6 +627,3 @@ class Artifact(models.Model):
 
     def __str__(self):
         return 'artifact-{0}'.format(self.id)
-
-
-post_save.connect(_extract_network_obs_id, sender=Artifact)
