@@ -1,5 +1,6 @@
 """Miscellaneous functions for SatNOGS DB"""
 import binascii
+import json
 import logging
 import math
 from datetime import datetime, timedelta
@@ -7,17 +8,80 @@ from decimal import Decimal
 
 from django.conf import settings
 from django.core.cache import cache
-from django.db.models import Count, Max, Q
+from django.db.models import Count, Max, OuterRef, Q, Subquery
+from django.db.models.functions import Substr
 from django.utils.timezone import make_aware
 from influxdb import InfluxDBClient
 from satnogsdecoders import __version__ as decoders_version
 from satnogsdecoders import decoder
 
-from db.base.models import DemodData, Mode, Satellite, Telemetry, Transmitter
+from db.base.models import DemodData, LatestTleSet, Mode, Satellite, \
+    Telemetry, Tle, Transmitter
 
 LOGGER = logging.getLogger('db')
 
 
+def remove_latest_tle_set(satellite_pk):
+    """Remove LatestTleSet for specific Satellite"""
+    LatestTleSet.objects.filter(satellite__pk=satellite_pk).delete()
+
+
+def update_latest_tle_sets(satellites=None):
+    """Update LatestTleSet for specific or all Satellites"""
+    if not satellites:
+        satellites = Satellite.objects.exclude(status__in=['re-entered', 'future'])
+
+    sub_subquery = Tle.objects.filter(satellite__pk__in=satellites).filter(
+        satellite=OuterRef('satellite'), tle_source=OuterRef('tle_source')
+    ).order_by('-updated')
+    subquery = Tle.objects.filter(pk=Subquery(sub_subquery.values('pk')[:1])
+                                  ).filter(satellite=OuterRef('satellite')
+                                           ).annotate(epoch=Max(Substr('tle1', 19, 14))
+                                                      ).order_by('-epoch')
+    tle_sets = Tle.objects.filter(pk=Subquery(subquery.values('pk')[:1]))
+
+    sub_subquery = Tle.objects.filter(satellite__pk__in=satellites).filter(
+        tle_source__in=settings.TLE_SOURCES_REDISTRIBUTABLE
+    ).filter(
+        satellite=OuterRef('satellite'), tle_source=OuterRef('tle_source')
+    ).order_by('-updated')
+    subquery = Tle.objects.filter(pk=Subquery(sub_subquery.values('pk')[:1])
+                                  ).filter(satellite=OuterRef('satellite')
+                                           ).annotate(epoch=Max(Substr('tle1', 19, 14))
+                                                      ).order_by('-epoch')
+    distributable_tle_sets = Tle.objects.filter(pk=Subquery(subquery.values('pk')[:1]))
+
+    for tle_set in tle_sets:
+        latest_tle_set, _ = LatestTleSet.objects.get_or_create(satellite=tle_set.satellite)
+        latest_tle_set.latest = tle_set
+        latest_tle_set.save(update_fields=['latest'])
+    for distributable_tle_set in distributable_tle_sets:
+        latest_tle_set, _ = LatestTleSet.objects.get_or_create(
+            satellite=distributable_tle_set.satellite
+        )
+        latest_tle_set.latest_distributable = distributable_tle_set
+        latest_tle_set.save(update_fields=['latest_distributable'])
+
+    LatestTleSet.objects.filter(latest__isnull=True, latest_distributable__isnull=True).delete()
+
+
+def get_tle_sources():
+    """Check for and return TLE custom sources"""
+    sources = {}
+    if settings.TLE_SOURCES_JSON:
+        try:
+            sources_json = json.loads(settings.TLE_SOURCES_JSON)
+            sources['sources'] = list(sources_json.items())
+        except json.JSONDecodeError as error:
+            print('TLE Sources JSON ignored as it is invalid: {}'.format(error))
+    if settings.SPACE_TRACK_USERNAME and settings.SPACE_TRACK_PASSWORD:
+        sources['spacetrack_config'] = {
+            'identity': settings.SPACE_TRACK_USERNAME,
+            'password': settings.SPACE_TRACK_PASSWORD
+        }
+    return sources
+
+
 def calculate_statistics():
     """Calculates statistics about the data we have in DB
 
@@ -222,12 +286,14 @@ def write_influx(json_obj):
     client.write_points(json_obj)
 
 
-def decode_data(norad, period=None):
+def decode_data(norad, period=None, handle=None):  # pylint: disable=R0912,R0914
     """Decode data for a satellite, with an option to limit the scope.
 
     :param norad: the NORAD ID of the satellite to decode data for
     :param period: if period exists, only attempt to decode the last 4 hours,
     otherwise attempt to decode everything
+    :param handle: if handle exists, we are trying to decode the currently
+    saved frame that got in
     """
     sat = Satellite.objects.get(norad_cat_id=norad)
     if not sat.telemetry_decoder_count:
@@ -238,7 +304,10 @@ def decode_data(norad, period=None):
         time_period = make_aware(time_period)
         data = DemodData.objects.filter(satellite__norad_cat_id=norad, timestamp__gte=time_period)
     else:
-        data = DemodData.objects.filter(satellite=sat)
+        if handle:
+            data = DemodData.objects.filter(id=handle)
+        else:
+            data = DemodData.objects.filter(satellite=sat)
     telemetry_decoders = Telemetry.objects.filter(satellite=sat)
 
     # iterate over DemodData objects
